--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      Screenshot-Saver - main class
-- @author    Valentin Schmidt
-- @version   1.0
--****************************************************************************

global $

-- config
property pAppName
property pFileFormat

-- Qt
property pTrayIcon
property pMenu
property pClipboard

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

	-- CONFIG
	me.pAppName = "ScreenshotSaver"
	me.pFileFormat = "png" -- bmp | jpg | png | tif
	
	-- CREATE TRAY-ICON
	me.pTrayIcon = $.Qt.SystemTrayIcon.new()
	me.pTrayIcon.setIcon(@("assets/trayicon.png"))
	me.pTrayIcon.setToolTip (me.pAppName)
		
	-- CREATE CONTEXT-MENU
	me.pMenu = $.Qt.Menu.new()
		
	action = $.Qt.Action.new(me.pMenu)
  action.setText("&About")
	action.connect(#triggered, me, #slotAbout)
  me.pMenu.addAction(action)
  	
  me.pMenu.addSeparator()
  	
	action = $.Qt.Action.new(me.pMenu)
  action.setText("&Open Image Folder")
	action.connect(#triggered, me, #slotOpen)
  me.pMenu.addAction(action)
  	
 	me.pMenu.addSeparator()
 		
	action = $.Qt.Action.new(me.pMenu)
  action.setText("&Quit")
	action.connect(#triggered, me, #slotQuit)
  me.pMenu.addAction(action)
  	
	me.pTrayIcon.setContextMenu(me.pMenu)

  -- WATCH SYSTEM CLIPBOARD FOR CHANGES
  me.pClipboard = $.Qt.Application.getClipboard()
  me.pClipboard.connect(#dataChanged, me, #slotClipboardDataChanged)
  	
	me.pTrayIcon.show()
  
	return me
end

--------------------------------------
-- @callback
--------------------------------------
on slotClipboardDataChanged(me)
	formats = me.pClipboard.formats()
	if formats.getPos("application/x-qt-image") then
		img = me.pClipboard.getImage()
		fn = $.path&"saved/capture_"&$.date.dateToString()&"."&me.pFileFormat
		ok = img.save(fn)
	end if
end

--------------------------------------
-- @callback
--------------------------------------
on slotAbout (me)
  msg = "<b>Screenshot Saver 1.0</b><br><br>"&\
  "Utility watching the system clipboard and automatically saving image data as files.<br>"&\
  "This application is based on <a href='http://www.qt.io/'>Qt</a> and <a href='http://valentin.dasdeck.com/lingo/lsw/'>LSW</a> and was written in <a href='http://en.wikipedia.org/wiki/Lingo_(programming_language)'>Lingo</a>.<br>"
  
	$.Qt.MessageBox.about(0, "About "&me.pAppName, msg)
end

--------------------------------------
-- @callback
-- @todo check if/make compatible with Win7 x64 etc
--------------------------------------
on slotOpen (me)
	explorer = getOSDirectory() & "\explorer.exe"
	imgPath = $.path&"saved"
	open imgPath, explorer
end

--------------------------------------
-- @callback
--------------------------------------
on slotQuit (me)
	halt()
end
