--!parent
--!encoding=utf-8

--****************************************************************************
-- @file			LSW CORE LIB: DATE
-- @author   	Valentin Schmidt
-- @version  	0.2
--****************************************************************************

property pDayList
property pMonthList

----------------------------------------
-- @constructor
----------------------------------------
on new (me, tScriptDir)
	me.pDayList = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
	me.pMonthList = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"]
	return me
end

----------------------------------------
-- @param {string[]} tDayList - list of weekday names
----------------------------------------
on setDayNames (me, tDayList)
	me.pDayList = tDayList
end

----------------------------------------
-- @param {string[]} tMonthList - list of month names
----------------------------------------
on setMonthNames (me, tMonthList)
  me.pMonthList = tMonthList
end

----------------------------------------
-- @return string day - the day as a string (eg. "Monday")
----------------------------------------
on mGetDayOfWeek (me, tDateObj, abbrevFlag)
  refDateObj = date(1905,1,2)
  currentDayNum = ((tDateObj - refDateObj) mod 7) +1
  currentDay = pDayList[currentDayNum]
  if abbrevFlag then currentDay = currentDay.char[1..3]
  return currentDay
end 

----------------------------------------
-- @return string month - the month as a string (eg. "January")
----------------------------------------
on mGetMonthFromDate (me, tDateObj, abbrevFlag)
  monthNum = tDateObj.month
  currentMonth = pMonthList[monthNum]
  if abbrevFlag then currentMonth = currentMonth.char[1..3]
  return currentMonth
end 

----------------------------------------
-- returns a lingo date object for specified date/time parameters (or for current date/time if called without parameter)
-- can be called with 0, 1, 2, 3, 4, 5 or 6 parameters
----------------------------------------
on newDate (me, tYear, tMonth, tDay, tHour, tMin, tSec)
  case (the paramCount) of
    0: return the systemDate
    1: return date(tYear)
    2: return date(tYear, tMonth)
    otherwise:
      d = date(tYear, tMonth, tDay)
      d.seconds = tHour*3600 + tMin*60 + tSec
      return d
  end case
end 

----------------------------------------
-- returns timestamp for date object (or for current date/time if parameter is omitted)
----------------------------------------
on dateToTimestamp (me, tDateObj)
  if voidP(tDateObj) then tDateObj = the systemdate
  ts = (tDateObj - date(1970,1,1))*86400 + tDateObj.seconds
  return ts
end

----------------------------------------
-- returns date object for given timestamp
----------------------------------------
on timestampToDate (me, ts)
  dateObj = date(1970,1,ts/86400)
  dateObj.seconds = ts mod 86400
  return dateObj
end

----------------------------------------
-- converts date object to string in YYYYMMDDHHMMII (or YYYY-MM-DD HH:MM:II) format
----------------------------------------
on dateToString (me, tDateObj, tSepFlag)
  if voidP(tDateObj) then tDateObj = the systemdate
  str = ""
  
  s = string(tDateObj.year)
  if s.length<4 then s = "0000".char[1..4-s.length]&s
  put s after str
  
  s = string(tDateObj.month)
  if s.length<2 then s = "0"&s
  put s after str
  
  s = string(tDateObj.day)
  if s.length<2 then s = "0"&s
  put s after str
  
  sec = tDateObj.seconds
  
  h = sec / 3600
  sec = sec mod 3600
  s = string(h)
  if s.length<2 then s = "0"&s
  put s after str
  
  m = sec / 60
  sec = sec mod 60
  s = string(m)
  if s.length<2 then s = "0"&s
  put s after str
  
  s = string(sec)
  if s.length<2 then s = "0"&s
  put s after str
  
  if tSepFlag then
  	put ":" after char 12 of str
  	put ":" after char 10 of str
		put " " after char 8 of str
  	put "-" after char 6 of str
  	put "-" after char 4 of str
  end if
  
  return str
end

----------------------------------------
-- converts YYYYMMDDHHMMII (or YYYY-MM-DD HH:MM:II) string to lingo date object
----------------------------------------
on stringToDate (me, tDateStr)
	if tDateStr contains "-" then
		if tDateStr.length>10 then
  		delete char 17 of tDateStr
  		delete char 14 of tDateStr
			delete char 11 of tDateStr
		end if
  	delete char 8 of tDateStr
  	delete char 5 of tDateStr
	end if
	tYear = integer(tDateStr.char[1..4])
	tMonth = integer(tDateStr.char[5..6])
	tDay = integer(tDateStr.char[7..8])
	tDateObj = date(tYear, tMonth, tDay)
	if tDateStr.length>8 then
		tHour = integer(tDateStr.char[9..10])
		tMin = integer(tDateStr.char[11..12])
		tSec = integer(tDateStr.char[13..14])
  	tDateObj.seconds = tHour*3600 + tMin*60 + tSec
  end if
  return tDateObj
end

----------------------------------------
-- returns number of seconds between 2 dates
----------------------------------------
on dateDiff (me, tDateObj1, tDateObj2)
  return dateToTimestamp(tDateObj1) - dateToTimestamp(tDateObj2)
end

----------------------------------------
-- 1=sunday...7=saturday
----------------------------------------
on dayOfWeek (me, tDateObj)
  return (tDateObj - date(1970,1,1) + 4) mod 7 + 1
end
