--!parent

--****************************************************************************
-- @file			LSW CORE LIB: FILE
-- @author   	Valentin Schmidt
-- @version  	1.0.0
--
-- @requires
-- - xtra("fileIO")
--
-- @desc
-- 
-- @todo
-- getCharSet object me -- returns the current charSet that is being used
-- setCharSet object me, string charSet -- to set the charSet
--****************************************************************************

----------------------------------------
-- @constructor
----------------------------------------
on new (me, tScriptDir)
	return me
end

--------------------------------------
-- opens file with specified mode, returns file pointer (xtra instance)
-- tMode: r, rb, w, wb, a, ab, rw, rwb, wr, wrb
--------------------------------------
on fopen (me, tFile, tMode)
  fp = xtra("fileIO").new()
  
  case (tMode) of
    "r","rb": fp.openFile(tFile, 1)
    "w","wb":
      --if not fileExists(tFile) then
      
			fp.openFile(tFile, 2)
			err = fp.status()
			if (err=0) then fp.delete()
			--else if (err and not (err = -37)) then return false
  	
      fp.createFile(tFile)
      fp.openFile(tFile, 2)
    "rw","wr","rwb","wrb": fp.openFile(tFile, 0)
    "a","ab":
      fp.openFile(tFile, 2)
      fp.setPosition(fp.getLength())
    otherwise:
      return 0
  end case
  
  if (fp.status()<>0) then fp = 0
  return fp
end

--------------------------------------
-- closes open file
--------------------------------------
on fclose (me, fp)
  fp.closeFile()
  fp = 0
end

--------------------------------------
-- reads n bytes from open file, returns string
--------------------------------------
on fread(me, fp, n)
  if voidP(n) then n = fp.getLength() - fp.getPosition()
  data = ""
  repeat with i = 1 to n
    c = fp.readChar()
    if fp.status()<>0 then exit repeat
    put numtochar(chartonum(c)) after data
  end repeat
  return data
end

--------------------------------------
-- writes string to open file
--------------------------------------
on fwrite (me, fp, str)
  fp.writeString(str)
end

--------------------------------------
-- returns position in open file
--------------------------------------
on ftell (me, fp)
  return fp.getPosition()
end

--------------------------------------
-- sets position in open file
-- whence: either integer or symbol
-- 0 or #SEEK_SET: set position to offset bytes. (default)
-- 1 or #SEEK_CUR: set position to current position plus offset.
-- 2 or #SEEK_END: set position to end of file plus offset.
--------------------------------------
on fseek (me, fp, pos, whence)
  if voidP(whence) then whence = 0
  else if ilk(whence)=#symbol then
    whence = [#SEEK_SET, #SEEK_CUR, #SEEK_END].getPos(whence)
    if whence>0 then whence=whence-1
  end if
  
  case (whence) of
    0: fp.setPosition(pos)  
    1: fp.setPosition(fp.getPosition() + pos) 
    2: fp.setPosition(fp.getLength() + pos) 
  end case
  
  return (fp.status()=0)
end

--------------------------------------
-- returns size of open file
--------------------------------------
on fsize (me, fp)
  return fp.getLength()
end

--**************************************
-- global functions, no file pointer passed
--**************************************

----------------------------------------
-- returns file size
----------------------------------------
on size (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if err then return err
  len = fp.getLength()
  fp.closeFile()
  fp = 0
  return len
end

----------------------------------------
-- truncates file
----------------------------------------
on truncate (me, tFile, tSize)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 0)
  data = fp.readByteArray(tSize)
  fp.delete()
  fp.createFile(tFile)
  fp.openFile(tFile, 2)
  fp.writeByteArray(data)
  err = fp.status()
  fp.closeFile()
  fp=0
  return (err=0)
end

----------------------------------------
-- deletes file
----------------------------------------
on delete (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 0)
  err = fp.status()
  if not (err) then err = fp.delete()
  fp = 0
  return err
end

----------------------------------------
-- is_file() ?
----------------------------------------
on exists (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  stat = fp.status()
  fp.closeFile()
  fp = 0
  -- 0: OK: item is a file
  -- -36: I/O Error: item is a folder
	-- -37: Bad File Name: does not exist
	return (stat=0)
end


----------------------------------------
-- reads whole file, returns string
----------------------------------------
on getString (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if (err) then return false
  ret = fp.readFile()
  fp.closeFile()
  fp = 0
  return ret
end

----------------------------------------
-- saves string as file
----------------------------------------
on putString (me, tFile, tString)
  fp = xtra("fileIO").new()  
  fp.openFile(tFile, 2)
  err = fp.status()
  if not (err) then fp.delete()
  else if (err and not (err = -37)) then return false
  fp.createFile(tFile)
  err = fp.status()
  if (err) then return false
  fp.openFile(tFile, 2)
  err = fp.status()
  if (err) then return false
  fp.writeString(tString)
  fp.closeFile()
  fp=0
  return true
end

----------------------------------------
-- returns file as ByteArray
----------------------------------------
on getBytes (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if (err) then return false
  data = fp.readByteArray(fp.getLength())
  fp.closeFile()
  fp = 0
  return data
end

----------------------------------------
-- saves ByteArray to file
----------------------------------------
on putBytes (me, tFile, tByteArray)
  fp = xtra("fileIO").new()  
  fp.openFile(tFile, 2)
  err = fp.status()
  if not (err) then fp.delete()
  else if (err and not (err = -37)) then return false
  fp.createFile(tFile)
  err = fp.status()
  if (err) then return false
  fp.openFile(tFile, 2)
  err = fp.status()
  if (err) then return false
  fp.writeByteArray(tByteArray)
  fp.closeFile()
  fp=0
  return true
end

----------------------------------------
-- display open file dialog
----------------------------------------
on selectFileOpen (me, tTitle, filterMask, defaultFileName)  
  fio = xtra("fileIO").new()  
  fio.setFilterMask(filterMask)
  fn = fio.displayOpen()
  fio = 0
  return string(fn)
end

----------------------------------------
-- display save file dialog
----------------------------------------
on selectFileSave (me, tTitle, filterMask, defaultFileName)  
  fio = xtra("fileIO").new()  
  fio.setFilterMask(filterMask)
  fn = fio.displaySave(tTitle, defaultFileName)
  fio = 0
  return string(fn)
end

----------------------------------------
-- select folder via save file dialog
----------------------------------------
on selectFolder (me, tTitle)
	if voidP(tTitle) then tTitle = "Select Folder"
  fio = new(xtra "fileIO")
  fn = fio.displaySave(tTitle, "*.*")
  err = fio.status()
  fio = 0
  if string(fn)=EMPTY then return ""
  if err<>0 then
		put "Save File Error:" && fio.error(err)
		--exit
		return ""
  end if
	pd = the last char of _movie.path
	od = _player.itemDelimiter
	_player.itemDelimiter = pd
	delete the last item of fn
	_player.itemDelimiter = od
	put pd after fn
	return fn
end
